@@ -25,10 +25,12 @@ from __future__ import print_function
 from __future__ import unicode_literals
 from dnf.comps import CompsQuery
 from dnf.i18n import _, P_, ucd
+from dnf.util import first
 from dnf.yum import history
 from dnf.yum import misc
 from dnf.yum import rpmsack
 from functools import reduce
+import collections
 import dnf.callback
 import dnf.comps
 import dnf.conf
@@ -74,7 +76,7 @@ class Base(object):
         self._closed = False
         self._conf = conf or self._setup_default_conf()
         self._goal = None
-        self._persistor = None
+        self.repo_persistor = None
         self._sack = None
         self._transaction = None
         self._ts = None
@@ -82,11 +84,13 @@ class Base(object):
         self._history = None
         self._tempfiles = set()
         self.ds_callback = dnf.callback.Depsolve()
-        self.group_persistor = None
+        self._group_persistor = None
         self.logging = dnf.logging.Logging()
         self._repos = dnf.repodict.RepoDict()
         self.rpm_probfilter = set([rpm.RPMPROB_FILTER_OLDPACKAGE])
         self.plugins = dnf.plugin.Plugins()
+        self.clean_tempfiles = False
+        self._tempfile_persistor = None
 
     def __enter__(self):
         return self
@@ -99,6 +103,7 @@ class Base(object):
         try:
             repo.load()
         except dnf.exceptions.RepoError as e:
+            repo.md_expire_cache()
             if repo.skip_if_unavailable is False:
                 raise
             logger.warning(_("%s, disabling."), e)
@@ -127,9 +132,8 @@ class Base(object):
         subst = conf.substitutions
         if 'releasever' not in subst:
             subst['releasever'] = \
-                dnf.rpm.detect_releasever(conf.installroot) or ''
-        suffix = dnf.conf.parser.substitute(dnf.const.CACHEDIR_SUFFIX, subst)
-        cache_dirs = dnf.conf.CliCache(conf.cachedir, suffix)
+                dnf.rpm.detect_releasever(conf.installroot)
+        cache_dirs = dnf.conf.CliCache(conf.cachedir)
 
         conf.cachedir = cache_dirs.cachedir
         return conf
@@ -167,13 +171,16 @@ class Base(object):
             except dnf.exceptions.MetadataError:
                 return False
 
-        if self._persistor:
+        if self.repo_persistor:
             expired = [r.id for r in self.repos.iter_enabled()
                        if check_expired(r)]
-            self._persistor.set_expired_repos(expired)
+            self.repo_persistor.set_expired_repos(expired)
 
-        if self.group_persistor:
-            self.group_persistor.save()
+        if self._group_persistor:
+            self._group_persistor.save()
+
+        if self._tempfile_persistor:
+            self._tempfile_persistor.save()
 
     @property
     def comps(self):
@@ -189,6 +196,10 @@ class Base(object):
     def goal(self):
         return self._goal
 
+    @property
+    def group_persistor(self):
+        return self._group_persistor
+
     @property
     def repos(self):
         return self._repos
@@ -219,7 +230,7 @@ class Base(object):
         self._transaction = value
 
     def activate_persistor(self):
-        self._persistor = dnf.persistor.RepoPersistor(self.conf.cachedir)
+        self.repo_persistor = dnf.persistor.RepoPersistor(self.conf.cachedir)
 
     def fill_sack(self, load_system_repo=True, load_available_repos=True):
         """Prepare the Sack and the Goal objects. :api."""
@@ -260,10 +271,25 @@ class Base(object):
             return
         logger.log(dnf.logging.DDEBUG, 'Cleaning up.')
         self._closed = True
+        self._tempfile_persistor = dnf.persistor.TempfilePersistor(
+            self.conf.cachedir)
 
-        if (not self.conf.keepcache and
-                not self.ts.isTsFlagSet(rpm.RPMTRANS_FLAG_TEST)):
-            self.clean_used_packages()
+        if self.clean_tempfiles:
+            # delete all packages from last unsuccessful transactions
+            self._tempfiles.update(
+                self._tempfile_persistor.get_saved_tempfiles())
+            self._tempfile_persistor.empty()
+        if not self.conf.keepcache:
+            if self.clean_tempfiles:
+                self.clean_used_packages()
+            else:
+                self._tempfile_persistor.tempfiles_to_add.update(
+                    self._tempfiles)
+        if self._tempfile_persistor.tempfiles_to_add:
+            logger.info(_("The downloaded packages were saved in cache "
+                          "till the next successful transaction."))
+            logger.info(_("You can remove cached packages by executing "
+                          "'%s'."), "dnf clean packages")
 
         # Do not trigger the lazy creation:
         if self._history is not None:
@@ -292,8 +318,8 @@ class Base(object):
             self._goal = None
             if self._sack is not None:
                 self._goal = dnf.goal.Goal(self._sack)
-            if self.group_persistor is not None:
-                self.group_persistor = self._activate_group_persistor()
+            if self._group_persistor is not None:
+                self._group_persistor = self._activate_group_persistor()
 
     def closeRpmDB(self):
         """Closes down the instances of rpmdb that could be open."""
@@ -343,12 +369,11 @@ class Base(object):
         self._ts = None
 
     def _activate_group_persistor(self):
-        return dnf.persistor.GroupPersistor(self.conf.persistdir)
+        return dnf.persistor.GroupPersistor(self.conf.persistdir, self._comps)
 
     def read_comps(self):
         """Create the groups object to access the comps metadata. :api"""
         timer = dnf.logging.Timer('loading comps')
-        self.group_persistor = self._activate_group_persistor()
         self._comps = dnf.comps.Comps()
 
         logger.log(dnf.logging.DDEBUG, 'Getting group metadata')
@@ -378,6 +403,7 @@ class Base(object):
                 msg = _('Failed to add groups file for repository: %s - %s')
                 logger.critical(msg % (repo.id, e))
 
+        self._group_persistor = self._activate_group_persistor()
         timer()
         return self._comps
 
@@ -473,7 +499,8 @@ class Base(object):
 
     def run_hawkey_goal(self, goal, allow_erasing):
         ret = goal.run(
-            allow_uninstall=allow_erasing, force_best=self.conf.best)
+            allow_uninstall=allow_erasing, force_best=self.conf.best,
+            ignore_weak_deps=(not self.conf.install_weak_deps))
         if self.conf.debug_solver:
             goal.write_debugdata('./debugdata')
         return ret
@@ -508,70 +535,72 @@ class Base(object):
             raise exc
         return got_transaction
 
-    def do_transaction(self, display=None):
+    def do_transaction(self, display=()):
         # :api
+        if not isinstance(display, collections.Sequence):
+            display = [display]
+        display = \
+            [dnf.yum.rpmtrans.LoggingTransactionDisplay()] + list(display)
 
-        persistor = self.group_persistor
+        persistor = self._group_persistor
         if persistor:
             persistor.commit()
 
         if not self.transaction:
             return
 
-        # save our ds_callback out
-        dscb = self.ds_callback
-        self.ds_callback = None
-        self.transaction.populate_rpm_ts(self.ts)
-
         logger.info(_('Running transaction check'))
-        msgs = self._run_rpm_check()
-        if msgs:
-            msg = _('Error: transaction check vs depsolve:')
-            logger.error(msg)
-            for msg in msgs:
-                logger.error(msg)
-            raise dnf.exceptions.TransactionCheckError(msg)
-
-        logger.info(_('Transaction check succeeded.'))
-
-        timer = dnf.logging.Timer('transaction test')
-        logger.info(_('Running transaction test'))
-        if not self.conf.diskspacecheck:
-            self.rpm_probfilter.add(rpm.RPMPROB_FILTER_DISKSPACE)
-
-        self.ts.order()  # order the transaction
-        self.ts.clean()  # release memory not needed beyond this point
-
-        testcb = dnf.yum.rpmtrans.RPMTransaction(self, test=True)
-        tserrors = self.ts.test(testcb)
-        del testcb
-
-        if len(tserrors) > 0:
-            errstring = _('Transaction check error:\n')
-            for descr in tserrors:
-                errstring += '  %s\n' % ucd(descr)
-
-            raise dnf.exceptions.Error(errstring + '\n' +
-                                       self._trans_error_summary(errstring))
-
-        logger.info(_('Transaction test succeeded.'))
-        timer()
-
-        # unset the sigquit handler
-        timer = dnf.logging.Timer('transaction')
-        # put back our depcheck callback
-        self.ds_callback = dscb
-        # setup our rpm ts callback
-        if display is None:
-            cb = dnf.yum.rpmtrans.RPMTransaction(self)
-        else:
-            cb = dnf.yum.rpmtrans.RPMTransaction(self, display=display)
-        if self.conf.debuglevel < 2:
-            cb.display.output = False
-
-        logger.info(_('Running transaction'))
         lock = dnf.lock.build_rpmdb_lock(self.conf.persistdir)
         with lock:
+            # save our ds_callback out
+            dscb = self.ds_callback
+            self.ds_callback = None
+            self.transaction.populate_rpm_ts(self.ts)
+
+            msgs = self._run_rpm_check()
+            if msgs:
+                msg = _('Error: transaction check vs depsolve:')
+                logger.error(msg)
+                for msg in msgs:
+                    logger.error(msg)
+                raise dnf.exceptions.TransactionCheckError(msg)
+
+            logger.info(_('Transaction check succeeded.'))
+
+            timer = dnf.logging.Timer('transaction test')
+            logger.info(_('Running transaction test'))
+            if not self.conf.diskspacecheck:
+                self.rpm_probfilter.add(rpm.RPMPROB_FILTER_DISKSPACE)
+
+            self.ts.order()  # order the transaction
+            self.ts.clean()  # release memory not needed beyond this point
+
+            testcb = dnf.yum.rpmtrans.RPMTransaction(self, test=True)
+            tserrors = self.ts.test(testcb)
+            del testcb
+
+            if len(tserrors) > 0:
+                errstring = _('Transaction check error:\n')
+                for descr in tserrors:
+                    errstring += '  %s\n' % ucd(descr)
+
+                raise dnf.exceptions.Error(errstring + '\n' +
+                                        self._trans_error_summary(errstring))
+
+            logger.info(_('Transaction test succeeded.'))
+            timer()
+
+            # unset the sigquit handler
+            timer = dnf.logging.Timer('transaction')
+            # put back our depcheck callback
+            self.ds_callback = dscb
+            # setup our rpm ts callback
+            cb = dnf.yum.rpmtrans.RPMTransaction(self, displays=display)
+            if self.conf.debuglevel < 2:
+                for display_ in cb.displays:
+                    display_.output = False
+
+            logger.info(_('Running transaction'))
             self._run_transaction(cb=cb)
         timer()
 
@@ -740,6 +769,7 @@ class Base(object):
             installed = rpmdb_sack.query().installed().nevra(
                 rpo.name, rpo.evr, rpo.arch)
             if len(installed) < 1:
+                tsi.op_type = dnf.transaction.FAIL
                 logger.critical(_('%s was supposed to be installed'
                                   ' but is not!' % rpo))
                 count = display_banner(rpo, count)
@@ -813,6 +843,7 @@ class Base(object):
             rpmdbv = rpmdb_sack.rpmdb_version(self.yumdb)
             self.history.end(rpmdbv, 0)
         timer()
+        self.clean_tempfiles = True
 
     def download_packages(self, pkglist, progress=None, callback_total=None):
         """Download the packages specified by the given list of packages. :api
@@ -912,10 +943,11 @@ class Base(object):
             hasgpgkey = not not repo.gpgkey
 
         if check:
-            ts = self.rpmconn.readonly_ts
+            root = self.conf.installroot
+            ts = dnf.rpm.transaction.initReadOnlyTransaction(root)
             sigresult = dnf.rpm.miscutils.checkSig(ts, po.localPkg())
             localfn = os.path.basename(po.localPkg())
-
+            del ts
             if sigresult == 0:
                 result = 0
                 msg = ''
@@ -1123,7 +1155,6 @@ class Base(object):
 
         # obsoleting packages (and what they obsolete)
         elif pkgnarrow == 'obsoletes':
-            self.conf.obsoletes = 1
             inst = q.installed()
             obsoletes = query_for_repo(
                 self.sack.query()).filter(obsoletes=inst)
@@ -1156,11 +1187,13 @@ class Base(object):
 
     def _add_comps_trans(self, trans):
         cnt = 0
+        clean_deps = self.conf.clean_requirements_on_remove
         attr_fn = ((trans.install, self._goal.install),
                    (trans.install_opt,
                     functools.partial(self._goal.install, optional=True)),
                    (trans.upgrade, self._goal.upgrade),
-                   (trans.remove, self._goal.erase))
+                   (trans.remove,
+                    functools.partial(self._goal.erase, clean_deps=clean_deps)))
 
         for (attr, fn) in attr_fn:
             for it in attr:
@@ -1186,17 +1219,21 @@ class Base(object):
             except AttributeError:
                 return 'unknown'
 
-        return dnf.comps.Solver(self.group_persistor, reason_fn)
+        return dnf.comps.Solver(self._group_persistor, self._comps, reason_fn)
 
-    def environment_install(self, env, types, exclude=None):
+    def environment_install(self, env_id, types, exclude=None, strict=True):
         solver = self.build_comps_solver()
         types = self._translate_comps_pkg_types(types)
-        trans = solver.environment_install(env, types, exclude or set())
+        trans = dnf.comps.install_or_skip(solver.environment_install,
+                                          env_id, types, exclude or set(),
+                                          strict)
+        if not trans:
+            return 0
         return self._add_comps_trans(trans)
 
-    def environment_remove(self, env):
+    def environment_remove(self, env_id):
         solver = self.build_comps_solver()
-        trans = solver.environment_remove(env)
+        trans = solver.environment_remove(env_id)
         return self._add_comps_trans(trans)
 
     _COMPS_TRANSLATION = {
@@ -1213,7 +1250,7 @@ class Base(object):
                 ret |= enum
         return ret
 
-    def group_install(self, grp, pkg_types, exclude=None):
+    def group_install(self, grp_id, pkg_types, exclude=None, strict=True):
         """Installs packages of selected group
         :param exclude: list of package name glob patterns
             that will be excluded from install set
@@ -1226,6 +1263,14 @@ class Base(object):
             else:
                 return (pattern,)
 
+        # compat: erase in 2.0.0
+        if isinstance(grp_id, dnf.comps.Environment) or \
+           isinstance(grp_id, dnf.comps.Group):
+                grp_id = grp_id.id
+                msg = "group_install() expects group id \
+                       instead of Comps object."
+                dnf.logging.depr(msg)
+
         exclude_pkgnames = None
         if exclude:
             nested_excludes = [_pattern_to_pkgname(p) for p in exclude]
@@ -1233,30 +1278,48 @@ class Base(object):
 
         solver = self.build_comps_solver()
         pkg_types = self._translate_comps_pkg_types(pkg_types)
-        trans = solver.group_install(grp, pkg_types, exclude_pkgnames)
+        trans = dnf.comps.install_or_skip(solver.group_install,
+                                          grp_id, pkg_types, exclude_pkgnames,
+                                          strict)
+        if not trans:
+            return 0
         logger.debug("Adding packages from group '%s': %s",
-                     grp.id, trans.install)
+                     grp_id, trans.install)
         return self._add_comps_trans(trans)
 
-    def env_group_install(self, patterns, types):
+    def env_group_install(self, patterns, types, strict=True):
         q = CompsQuery(self.comps, self.group_persistor,
                        CompsQuery.ENVIRONMENTS | CompsQuery.GROUPS,
                        CompsQuery.AVAILABLE | CompsQuery.INSTALLED)
-        try:
-            res = q.get(*patterns)
-        except dnf.exceptions.CompsError as err:
-            logger.error("Warning: %s", ucd(err))
-            return 0
-        cnt = dnf.comps.install_or_skip(self.environment_install,
-                                        res.environments, types)
-        cnt += dnf.comps.install_or_skip(self.group_install,
-                                         res.groups, types)
+        cnt = 0
+        done = True
+        for pattern in patterns:
+            try:
+                res = q.get(pattern)
+            except dnf.exceptions.CompsError as err:
+                logger.error("Warning: %s", ucd(err))
+                done = False
+                continue
+            for group_id in res.groups:
+                cnt += self.group_install(group_id, types, strict=strict)
+            for env_id in res.environments:
+                cnt += self.environment_install(env_id, types, strict=strict)
+        if not done and strict:
+            raise dnf.exceptions.Error(_('Nothing to do.'))
         return cnt
 
-    def group_remove(self, grp):
+    def group_remove(self, grp_id):
         # :api
+        # compat: erase in 2.0.0
+        if isinstance(grp_id, dnf.comps.Environment) or \
+           isinstance(grp_id, dnf.comps.Group):
+                msg = "group_remove() expects group id \
+                       instead of Comps object."
+                dnf.logging.depr(msg)
+                grp_id = grp_id.id
+
         solver = self.build_comps_solver()
-        trans = solver.group_remove(grp)
+        trans = solver.group_remove(grp_id)
         return self._add_comps_trans(trans)
 
     def env_group_remove(self, patterns):
@@ -1267,7 +1330,7 @@ class Base(object):
             res = q.get(*patterns)
         except dnf.exceptions.CompsError as err:
             logger.error("Warning: %s", ucd(err))
-            return 0
+            raise dnf.exceptions.Error(_('No groups marked for removal.'))
         cnt = 0
         for env in res.environments:
             cnt += self.environment_remove(env)
@@ -1275,10 +1338,18 @@ class Base(object):
             cnt += self.group_remove(grp)
         return cnt
 
-    def group_upgrade(self, grp):
+    def group_upgrade(self, grp_id):
         # :api
+        # compat: erase in 2.0.0
+        if isinstance(grp_id, dnf.comps.Environment) or \
+           isinstance(grp_id, dnf.comps.Group):
+                msg = "group_upgrade() expects group id \
+                       instead of Comps object."
+                dnf.logging.depr(msg)
+                grp_id = grp_id.id
+
         solver = self.build_comps_solver()
-        trans = solver.group_upgrade(grp)
+        trans = solver.group_upgrade(grp_id)
         return self._add_comps_trans(trans)
 
     def gpgKeyCheck(self):
@@ -1311,7 +1382,7 @@ class Base(object):
             del fo
             return 1
 
-    def install(self, pkg_spec, reponame=None):
+    def install(self, pkg_spec, reponame=None, strict=True):
         """Mark package(s) given by pkg_spec and reponame for installation.:api
         """
 
@@ -1328,7 +1399,7 @@ class Base(object):
             for i in already_inst:
                 _msg_installed(i)
             for a in available:
-                self._goal.install(a)
+                self._goal.install(a, optional=(not strict))
             return len(available)
         elif self.conf.multilib_policy == "best":
             sltrs = subj.get_best_selectors(self.sack)
@@ -1345,7 +1416,7 @@ class Base(object):
                 if already_inst:
                     for package in already_inst:
                         _msg_installed(package)
-                self._goal.install(select=sltr)
+                self._goal.install(select=sltr, optional=(not strict))
             return 1
         return 0
 
@@ -1386,14 +1457,14 @@ class Base(object):
             logger.warning(msg)
             return 0
 
-    def package_install(self, pkg):
+    def package_install(self, pkg, strict=True):
         # :api
         q = self.sack.query().nevra(pkg.name, pkg.evr, pkg.arch)
         already_inst, _ = self._query_matches_installed(q)
         if pkg in already_inst:
             _msg_installed(pkg)
         else:
-            self._goal.install(pkg)
+            self._goal.install(pkg, optional=(not strict))
         return 1
 
     def package_reinstall(self, pkg):
@@ -1430,6 +1501,9 @@ class Base(object):
 
     def upgrade(self, pkg_spec, reponame=None):
         # :api
+        def is_installed_by_name(pkg_name):
+            return first(self.sack.query().installed().filter(name=pkg_name))
+        wildcard = True if dnf.util.is_glob_pattern(pkg_spec) else False
         sltrs = dnf.subject.Subject(pkg_spec).get_best_selectors(self.sack)
         match = reduce(lambda x, y: y.matches() or x, sltrs, [])
         if match:
@@ -1437,6 +1511,12 @@ class Base(object):
             for sltr in sltrs:
                 if not sltr.matches():
                     continue
+                pkg_name = sltr.matches()[0].name
+                if not is_installed_by_name(pkg_name):
+                    if not wildcard: # wildcard shouldn't print not installed packages
+                        msg = _("Package %s not installed, cannot update it.")
+                        logger.warning(msg, pkg_name)
+                    continue
                 if reponame is not None:
                     sltr = sltr.set(reponame=reponame)
                 self._goal.upgrade(select=sltr)
@@ -1553,7 +1633,7 @@ class Base(object):
         subj = dnf.subject.Subject(pkg_spec)
         q = subj.get_best_query(self.sack)
         installed = sorted(q.installed())
-        installed_pkg = dnf.util.first(installed)
+        installed_pkg = first(installed)
         if installed_pkg is None:
             available_pkgs = q.available()
             if available_pkgs:
@@ -1565,7 +1645,7 @@ class Base(object):
         arch = installed_pkg.arch
         q = self.sack.query().filter(name=installed_pkg.name, arch=arch)
         avail = [pkg for pkg in q.downgrades() if pkg < installed_pkg]
-        avail_pkg = dnf.util.first(sorted(avail, reverse=True))
+        avail_pkg = first(sorted(avail, reverse=True))
         if avail_pkg is None:
             return 0
 
@@ -1585,7 +1665,8 @@ class Base(object):
 
         q = subj._nevra_to_filters(self.sack.query(), nevra)
         available_pkgs = q.available()
-        if not self.sack.query().filter(name=nevra.name).installed():
+        test_q = dnf.subject.Subject(nevra.name).get_best_query(self.sack)
+        if not test_q.installed():
             raise dnf.exceptions.PackagesNotInstalledError(
                 'no package matched', pkg_spec, available_pkgs)
         downgrade_pkgs = available_pkgs.downgrades().latest()
@@ -1629,7 +1710,7 @@ class Base(object):
                 raise dnf.exceptions.PackagesNotAvailableError(
                     'no package matched', old_nevra)
             assert len(news) == 1
-            self._transaction.add_upgrade(dnf.util.first(olds), news[0], None)
+            self._transaction.add_upgrade(first(olds), news[0], None)
             for obsoleted_nevra in obsoleted_nevras:
                 handle_erase(obsoleted_nevra)
 
@@ -1640,7 +1721,7 @@ class Base(object):
                 raise dnf.exceptions.PackagesNotAvailableError(
                     'no package matched', old_nevra)
             self._transaction.add_install(
-                dnf.util.first(pkgs), None, 'history')
+                first(pkgs), None, 'history')
 
         def handle_install(new_nevra, obsoleted_nevras):
             """Handle an installed package."""
@@ -1670,7 +1751,7 @@ class Base(object):
                     assert len(obsoleteds_) == 1
                     obsoleteds.append(obsoleteds_[0])
             assert len(news) == 1
-            self._transaction.add_reinstall(dnf.util.first(olds), news[0],
+            self._transaction.add_reinstall(first(olds), news[0],
                                             obsoleteds)
 
         def handle_upgrade(new_nevra, old_nevra, obsoleted_nevras):
@@ -1685,7 +1766,7 @@ class Base(object):
                     'no package matched', old_nevra)
             assert len(news) == 1
             self._transaction.add_downgrade(
-                dnf.util.first(olds), news[0], None)
+                first(olds), news[0], None)
             for obsoleted_nevra in obsoleted_nevras:
                 handle_erase(obsoleted_nevra)
 
