@@ -28,6 +28,7 @@ from . import output
 from dnf.cli import CliError
 from dnf.i18n import ucd, _
 
+import collections
 import datetime
 import dnf
 import dnf.cli.commands
@@ -39,6 +40,7 @@ import dnf.cli.commands.remove
 import dnf.cli.commands.group
 import dnf.cli.commands.install
 import dnf.cli.commands.makecache
+import dnf.cli.commands.mark
 import dnf.cli.commands.reinstall
 import dnf.cli.commands.repolist
 import dnf.cli.commands.search
@@ -77,7 +79,7 @@ def _add_pkg_simple_list_lens(data, pkg, indent=''):
         This "knows" about simpleList and printVer. """
     na = len(pkg.name) + 1 + len(pkg.arch) + len(indent)
     ver = len(pkg.evr)
-    rid = len(pkg.reponame)
+    rid = len(pkg.from_repo)
     for (d, v) in (('na', na), ('ver', ver), ('rid', rid)):
         data[d].setdefault(v, 0)
         data[d][v] += 1
@@ -101,9 +103,7 @@ def _list_cmd_calc_columns(output, ypl):
 
 
 def cachedir_fit(conf):
-    subst = conf.substitutions
-    suffix = dnf.conf.parser.substitute(dnf.const.CACHEDIR_SUFFIX, subst)
-    cli_cache = dnf.conf.CliCache(conf.cachedir, suffix)
+    cli_cache = dnf.conf.CliCache(conf.cachedir)
     return cli_cache.cachedir, cli_cache.system_cachedir
 
 
@@ -139,6 +139,7 @@ class BaseCli(dnf.Base):
     """This is the base class for yum cli."""
 
     def __init__(self, conf=None):
+        self.cmd_conf = CmdConf()
         conf = conf or dnf.conf.Conf()
         super(BaseCli, self).__init__(conf=conf)
         self.output = output.Output(self, self.conf)
@@ -148,17 +149,18 @@ class BaseCli(dnf.Base):
             return None
         return self.group_persistor.diff()
 
-    def do_transaction(self):
+    def do_transaction(self, display=()):
         """Take care of package downloading, checking, user
         confirmation and actually running the transaction.
 
+        :param display: `rpm.callback.TransactionProgress` object(s)
         :return: a numeric return code, and optionally a list of
            errors.  A negative return code indicates that errors
            occurred in the pre-transaction checks
         """
 
         grp_diff = self._groups_diff()
-        grp_str = self.output.list_group_transaction(self.comps, grp_diff)
+        grp_str = self.output.list_group_transaction(self.comps, self.group_persistor, grp_diff)
         if grp_str:
             logger.info(grp_str)
         trans = self.transaction
@@ -192,7 +194,7 @@ class BaseCli(dnf.Base):
             else:
                 self.output.reportDownloadSize(downloadpkgs, install_only)
 
-        if trans or grp_diff:
+        if trans or (grp_diff and not grp_diff.empty()):
             # confirm with user
             if self._promptWanted():
                 if self.conf.assumeno or not self.output.userconfirm():
@@ -211,11 +213,18 @@ class BaseCli(dnf.Base):
             except dnf.exceptions.DownloadError as e:
                 specific = dnf.cli.format.indent_block(ucd(e))
                 errstring = _('Error downloading packages:\n%s') % specific
+                # setting the new line to prevent next chars being eaten up by carriage returns
+                print()
                 raise dnf.exceptions.Error(errstring)
             # Check GPG signatures
             self.gpgsigcheck(downloadpkgs)
 
-        display = output.CliTransactionDisplay()
+        if self.cmd_conf.downloadonly:
+            return
+
+        if not isinstance(display, collections.Sequence):
+            display = [display]
+        display = [output.CliTransactionDisplay()] + list(display)
         super(BaseCli, self).do_transaction(display)
         if trans:
             msg = self.output.post_transaction_output(trans)
@@ -339,6 +348,7 @@ class BaseCli(dnf.Base):
         oldcount = self._goal.req_length()
 
         for arg in userlist:
+            wildcard = True if dnf.util.is_glob_pattern(arg) else False
             if arg.endswith('.rpm'):
                 pkg = self.add_remote_rpm(arg)
                 self.package_downgrade(pkg)
@@ -352,8 +362,10 @@ class BaseCli(dnf.Base):
                 logger.info(msg, self.output.term.MODE['bold'], arg,
                                  self.output.term.MODE['normal'])
             except dnf.exceptions.PackagesNotInstalledError as err:
-                for pkg in err.packages:
-                    logger.info(_('No match for available package: %s'), pkg)
+                if not wildcard:
+                # glob pattern should not match not installed packages -> ignore error
+                    for pkg in err.packages:
+                        logger.info(_('No match for available package: %s'), pkg)
             except dnf.exceptions.MarkingError:
                 assert False
         cnt = self._goal.req_length() - oldcount
@@ -700,6 +712,7 @@ class Cli(object):
         self.register_command(dnf.cli.commands.group.GroupCommand)
         self.register_command(dnf.cli.commands.install.InstallCommand)
         self.register_command(dnf.cli.commands.makecache.MakeCacheCommand)
+        self.register_command(dnf.cli.commands.mark.MarkCommand)
         self.register_command(dnf.cli.commands.reinstall.ReinstallCommand)
         self.register_command(dnf.cli.commands.remove.RemoveCommand)
         self.register_command(dnf.cli.commands.repolist.RepoListCommand)
@@ -723,6 +736,15 @@ class Cli(object):
 
     def _configure_repos(self, opts):
         self.base.read_all_repos(self.repo_setopts)
+        if opts.repofrompath:
+            for label, path in opts.repofrompath.items():
+                if path[0] == '/':
+                    path = 'file://' + path
+                repofp = dnf.repo.Repo(label, self.base.conf.cachedir)
+                repofp.baseurl = path
+                self.base.repos.add(repofp)
+                logger.info(_("Added %s repo from %s") % (label, path))
+
         # Process repo enables and disables in order
         try:
             for (repo, operation) in opts.repos_ed:
@@ -744,7 +766,7 @@ class Cli(object):
                 repo.gpgcheck = False
                 repo.repo_gpgcheck = False
 
-        for rid in self.base._persistor.get_expired_repos():
+        for rid in self.base.repo_persistor.get_expired_repos():
             repo = self.base.repos.get(rid)
             if repo:
                 repo.md_expire_cache()
@@ -795,7 +817,7 @@ class Cli(object):
                     for repo in repos:
                         logger.debug(_("%s: using metadata from %s."),
                                      repo.id,
-                                     time.ctime(repo.metadata.timestamp))
+                                     time.ctime(repo.metadata.md_timestamp))
                     logger.info(_("Last metadata expiration check performed "
                                   "%s ago on %s."),
                                 datetime.timedelta(seconds=age),
@@ -1013,6 +1035,8 @@ class Cli(object):
         if opts.debugsolver:
             self.base.conf.debug_solver = True
 
+        self.base.cmd_conf.downloadonly = opts.downloadonly
+
         self.base.plugins.run_config()
 
     def check(self):
@@ -1027,10 +1051,6 @@ class Cli(object):
         conf.read(path)
         if releasever is None:
             releasever = dnf.rpm.detect_releasever(root)
-            if releasever is None:
-                msg = _('releasever not given and can not be detected '
-                        'from the installroot.')
-                raise dnf.exceptions.ConfigError(msg)
         conf.releasever = releasever
         subst = conf.substitutions
         subst.update_from_etc(root)
@@ -1045,12 +1065,6 @@ class Cli(object):
 
         self.base.logging.setup_from_dnf_conf(conf)
 
-        # repos are ver/arch specific so add $basearch/$releasever
-        conf._repos_persistdir = os.path.normpath(
-            '%s/repos/%s/%s/' % (conf.persistdir,
-                                 subst.get('basearch', '$basearch'),
-                                 subst.get('releasever', '$releasever')))
-
         timer()
         return conf
 
@@ -1078,3 +1092,8 @@ class Cli(object):
 
     def print_usage(self):
         return self.optparser.print_usage()
+
+
+class CmdConf(object):
+    """Class for storing nonpersistent configuration"""
+    downloadonly = False
