@@ -1,5 +1,5 @@
 # Copyright 2005 Duke University
-# Copyright (C) 2012-2013  Red Hat, Inc.
+# Copyright (C) 2012-2015  Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ def _make_lists(transaction):
     def tsi_cmp_key(tsi):
         return str(tsi.active)
 
-    TYPES = ('downgraded', 'erased', 'installed', 'reinstalled', 'upgraded')
+    TYPES = ('downgraded', 'erased', 'installed', 'reinstalled', 'upgraded', 'failed')
     b = dnf.util.Bunch()
     for ttype in TYPES:
         b[ttype] = []
@@ -65,6 +65,9 @@ def _make_lists(transaction):
             b.reinstalled.append(tsi)
         elif tsi.op_type == dnf.transaction.UPGRADE:
             b.upgraded.append(tsi)
+        elif tsi.op_type == dnf.transaction.FAIL:
+            b.failed.append(tsi)
+
     for ttype in TYPES:
         b[ttype].sort(key=tsi_cmp_key)
     return b
@@ -370,7 +373,7 @@ class Output(object):
             columns = (-40, -22, -16) # Old default
         na = '%s%s.%s' % (indent, pkg.name, pkg.arch)
         hi_cols = [highlight, 'normal', 'normal']
-        columns = zip((na, pkg.evr, pkg.reponame), columns, hi_cols)
+        columns = zip((na, pkg.evr, pkg.from_repo), columns, hi_cols)
         print(self.fmtColumns(columns))
 
     def simpleEnvraList(self, pkg, ui_overflow=False,
@@ -901,7 +904,7 @@ class Output(object):
         if not error:
             logger.info(_("Installed size: %s"), format_number(totsize))
 
-    def list_group_transaction(self, comps, diff):
+    def list_group_transaction(self, comps, prst, diff):
         if not diff:
             return None
 
@@ -918,7 +921,7 @@ class Output(object):
             out.append(_('Marking removed:'))
         for grp_id in diff.removed_groups:
             pkgs = list(diff.removed_packages(grp_id))
-            grp_name = comps.group_by_id(grp_id).ui_name
+            grp_name = prst.group(grp_id).ui_name
             rows.extend(_spread_in_columns(4, grp_name, pkgs))
 
         if rows:
@@ -928,11 +931,42 @@ class Output(object):
             out[0:0] = self._banner(col_data, (_('Group'), _('Packages'), '', ''))
         return '\n'.join(out)
 
+    def _skipped_conflicts(self):
+        """returns set of packages that would be additionally installed
+           when --best and --allowerasing is set"""
+        def is_better_version(same_name_pkgs):
+            pkg1, pkg2 = same_name_pkgs
+            if not pkg2 or (pkg1 and pkg1 > pkg2):
+                return False
+            return True
+
+        return set(map(lambda t: t[1], filter(is_better_version,
+                   self.base._goal.best_run_diff())))
+
+    def _skipped_broken_deps(self, skipped_conflicts):
+        """returns set of packages that are available updates
+           but cannot be upgraded"""
+        goal_diff = self.base._goal.available_updates_diff(
+            self.base.sack.query())
+        if skipped_conflicts:
+            goal_diff -= skipped_conflicts
+        return goal_diff
+
     def list_transaction(self, transaction):
         """Return a string representation of the transaction in an
         easy-to-read format.
         """
 
+        forward_actions = {
+            hawkey.UPGRADE,
+            hawkey.UPGRADE_ALL,
+            hawkey.DISTUPGRADE,
+            hawkey.DISTUPGRADE_ALL,
+            hawkey.DOWNGRADE,
+            hawkey.INSTALL
+        }
+        skipped_conflicts = []
+
         if transaction is None:
             return None
 
@@ -944,7 +978,7 @@ class Output(object):
         def _add_line(lines, data, a_wid, po, obsoletes=[]):
             (n, a, e, v, r) = po.pkgtup
             evr = po.evr
-            repoid = po.reponame
+            repoid = po.from_repo
             size = format_number(po.size)
 
             if a is None: # gpgkeys are weird
@@ -978,6 +1012,25 @@ class Output(object):
 
             pkglist_lines.append((action, lines))
 
+        # show skipped conflicting packages
+        if not self.conf.best and forward_actions & self.base._goal.actions:
+            lines = []
+            skipped_conflicts = self._skipped_conflicts()
+            for pkg in sorted(skipped_conflicts):
+                a_wid = _add_line(lines, data, a_wid, pkg, [])
+            skip_str = _("Skipping packages with conflicts:\n"
+                         "(add '%s' to command line "
+                         "to force their upgrade)") % "--best --allowerasing"
+            pkglist_lines.append((skip_str, lines))
+
+        # show skipped packages with broken dependencies
+        if hawkey.UPGRADE_ALL in self.base._goal.actions:
+            lines = []
+            for pkg in sorted(self._skipped_broken_deps(skipped_conflicts)):
+                a_wid = _add_line(lines, data, a_wid, pkg, [])
+            skip_str = _("Skipping packages with broken dependencies")
+            pkglist_lines.append((skip_str, lines))
+
         if not data['n']:
             return u''
         else:
@@ -1112,7 +1165,8 @@ Transaction Summary
                                (_('Removed'), list_bunch.erased),
                                (_('Installed'), list_bunch.installed),
                                (_('Upgraded'), list_bunch.upgraded),
-                               (_('Downgraded'), list_bunch.downgraded)]:
+                               (_('Downgraded'), list_bunch.downgraded),
+                               (_('Failed'), list_bunch.failed)]:
             if not tsis:
                 continue
             msgs = []
@@ -2066,10 +2120,10 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
         self.output = True
 
         # for a progress bar
-        self.mark = "#"
+        self.mark = "="
         self.marks = 22
 
-    def event(self, package, action, te_current, te_total, ts_current, ts_total):
+    def progress(self, package, action, ti_done, ti_total, ts_done, ts_total):
         """Output information about an rpm operation.  This may
         include a text progress bar.
 
@@ -2077,11 +2131,11 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
         :param action: the type of action that is taking place.  Valid
            values are given by
            :func:`rpmtrans.LoggingTransactionDisplay.action.keys()`
-        :param te_current: a number representing the amount of work
+        :param ti_done: a number representing the amount of work
            already done in the current transaction
-        :param te_total: a number representing the total amount of work
+        :param ti_total: a number representing the total amount of work
            to be done in the current transaction
-        :param ts_current: the number of the current transaction in
+        :param ts_done: the number of the current transaction in
            transaction set
         :param ts_total: the total number of transactions in the
            transaction set
@@ -2094,12 +2148,12 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
 
         pkgname = ucd(package)
         self.lastpackage = package
-        if te_total == 0:
+        if ti_total == 0:
             percent = 0
         else:
-            percent = (te_current*long(100))//te_total
-        self._out_event(te_current, te_total, ts_current, ts_total,
-                        percent, process, pkgname, wid1)
+            percent = (ti_done*long(100))//ti_total
+        self._out_progress(ti_done, ti_total, ts_done, ts_total,
+                           percent, process, pkgname, wid1)
 
     def _max_action_width(self):
         if not hasattr(self, '_max_action_wid_cache'):
@@ -2112,10 +2166,10 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
         wid1 = self._max_action_wid_cache
         return wid1
 
-    def _out_event(self, te_current, te_total, ts_current, ts_total,
-                   percent, process, pkgname, wid1):
-        if self.output and (sys.stdout.isatty() or te_current == te_total):
-            (fmt, wid1, wid2) = self._makefmt(percent, ts_current, ts_total,
+    def _out_progress(self, ti_done, ti_total, ts_done, ts_total,
+                      percent, process, pkgname, wid1):
+        if self.output and (sys.stdout.isatty() or ti_done == ti_total):
+            (fmt, wid1, wid2) = self._makefmt(percent, ts_done, ts_total,
                                               progress=sys.stdout.isatty(),
                                               pkgname=pkgname, wid1=wid1)
             pkgname = ucd(pkgname)
@@ -2125,7 +2179,7 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
                 sys.stdout.write(msg)
                 sys.stdout.flush()
                 self.lastmsg = msg
-            if te_current == te_total:
+            if ti_done == ti_total:
                 print(" ")
 
     def scriptout(self, msgs):
@@ -2137,12 +2191,12 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
             sys.stdout.write(ucd(msgs))
             sys.stdout.flush()
 
-    def _makefmt(self, percent, ts_current, ts_total, progress=True,
+    def _makefmt(self, percent, ts_done, ts_total, progress=True,
                  pkgname=None, wid1=15):
         l = len(str(ts_total))
         size = "%s.%s" % (l, l)
         fmt_done = "%" + size + "s/%" + size + "s"
-        done = fmt_done % (ts_current, ts_total)
+        done = fmt_done % (ts_done, ts_total)
 
         #  This should probably use TerminLine, but we don't want to dep. on
         # that. So we kind do an ok job by hand ... at least it's dynamic now.
@@ -2173,24 +2227,24 @@ class CliTransactionDisplay(LoggingTransactionDisplay):
             fmt = "\r  %s: %s   " + done
             wid2 = full_pnl
         elif progress:
-            bar = fmt_bar % (self.mark * int(marks * (percent / 100.0)), )
+            if marks > 5:
+                bar = fmt_bar % (self.mark * int(marks * (percent / 100.0)), )
+            else:
+                bar = ""
             fmt = "\r  %s: %s " + bar + " " + done
             wid2 = pnl
         elif percent == 100:
             fmt = "  %s: %s   " + done
             wid2 = full_pnl
         else:
-            bar = fmt_bar % (self.mark * marks, )
+            if marks > 5:
+                bar = fmt_bar % (self.mark * marks, )
+            else:
+                bar = ""
             fmt = "  %s: %s " + bar + " " + done
             wid2 = pnl
         return fmt, wid1, wid2
 
-    def verify_tsi_package(self, pkg, count, total):
-        percent = 100
-        process = _('Verifying')
-        wid1 = max(exact_width(process), self._max_action_width())
-        self._out_event(100, 100, count, total, percent, process, ucd(pkg), wid1)
-
 def progressbar(current, total, name=None):
     """Output the current status to the terminal using a simple
     text progress bar consisting of 50 # marks.
