@@ -33,6 +33,7 @@ import dnf.util
 import dnf.yum.config
 import dnf.yum.misc
 import functools
+import hashlib
 import hawkey
 import logging
 import librepo
@@ -138,6 +139,7 @@ def download_payloads(payloads, drpm):
         if err == 'Already downloaded':
             errs.skipped.add(pkg)
             continue
+        pkg.repo.md_expire_cache()
         errs.irrecoverable[pkg] = [err]
 
     return errs
@@ -163,13 +165,14 @@ class _DetailedLibrepoError(Exception):
 
 
 class _Handle(librepo.Handle):
-    def __init__(self, gpgcheck, max_mirror_tries):
+    def __init__(self, gpgcheck, max_mirror_tries, max_parallel_downloads=None):
         super(_Handle, self).__init__()
         self.gpgcheck = gpgcheck
         self.maxmirrortries = max_mirror_tries
         self.interruptible = True
         self.repotype = librepo.LR_YUMREPO
         self.useragent = dnf.const.USER_AGENT
+        self.maxparalleldownloads = max_parallel_downloads
         self.yumdlist = [
             "primary", "filelists", "prestodelta", "group_gz", "updateinfo"]
 
@@ -452,7 +455,14 @@ class Repo(dnf.yum.config.RepoConf):
 
     @property
     def cachedir(self):
-        return os.path.join(self.basecachedir, self.id)
+        url = self.metalink or self.mirrorlist \
+              or (self.baseurl and self.baseurl[0])
+        if url:
+            digest = hashlib.sha256(url.encode('utf8')).hexdigest()[:16]
+            repodir = "%s-%s" % (self.id, digest)
+        else:
+            repodir = self.id
+        return os.path.join(self.basecachedir, repodir)
 
     @property
     def filelists_fn(self):
@@ -581,7 +591,8 @@ class Repo(dnf.yum.config.RepoConf):
         return self._handle_new_remote(self.pkgdir, mirror_setup=False)
 
     def _handle_new_remote(self, destdir, mirror_setup=True):
-        h = _Handle(self.repo_gpgcheck, self.max_mirror_tries)
+        h = _Handle(self.repo_gpgcheck, self.max_mirror_tries,
+                    self.max_parallel_downloads)
         h.varsub = _subst2tuples(self.substitutions)
         h.destdir = destdir
         self._set_ip_resolve(h)
@@ -611,6 +622,14 @@ class Repo(dnf.yum.config.RepoConf):
                 userpwd += ":" + self.password
             h.setopt(librepo.LRO_USERPWD, userpwd)
 
+        # setup ssl stuff
+        if self.sslcacert:
+            h.setopt(librepo.LRO_SSLCACERT, self.sslcacert)
+        if self.sslclientcert:
+            h.setopt(librepo.LRO_SSLCLIENTCERT, self.sslclientcert)
+        if self.sslclientkey:
+            h.setopt(librepo.LRO_SSLCLIENTKEY, self.sslclientkey)
+
         # setup download progress
         h.progresscb = self._md_pload._progress_cb
         self._md_pload.fm_running = False
@@ -619,6 +638,7 @@ class Repo(dnf.yum.config.RepoConf):
         # apply repo options
         h.maxspeed = self.throttle if type(self.throttle) is int \
                      else int(self.bandwidth * self.throttle)
+        h.setopt(librepo.LRO_PROXYAUTH, True)
         h.proxy = self.proxy
         h.lowspeedlimit = self.minrate
         h.lowspeedtime = self.timeout
