@@ -758,7 +758,7 @@ class YumConf(BaseConfig):
                                           names_of_0=["0", "<off>"])
     tsflags = ListOption() # :api
 
-    assumeyes = BoolOption(False)
+    assumeyes = BoolOption(False)  # :api
     assumeno = BoolOption(False)
     defaultyes = BoolOption(False)
     alwaysprompt = BoolOption(True)
@@ -772,18 +772,20 @@ class YumConf(BaseConfig):
     enablegroups = BoolOption(True)
 
     bandwidth = BytesOption(0)
-    minrate = BytesOption(0)
+    minrate = BytesOption(1000)
     ip_resolve = CaselessSelectionOption(
             allowed=('ipv4', 'ipv6', 'whatever'),
             mapper={'4': 'ipv4', '6': 'ipv6'})
     throttle = ThrottleOption(0)
     timeout = SecondsOption(120)
+    max_parallel_downloads = IntOption(None, range_min=1)
 
     metadata_expire = SecondsOption(60 * 60 * 48)    # 48 hours
     metadata_timer_sync = SecondsOption(60 * 60 * 3) #  3 hours
     disable_excludes = ListOption()
     multilib_policy = SelectionOption('best', ('best', 'all')) # :api
     best = BoolOption(False) # :api
+    install_weak_deps = BoolOption(True)
     bugtracker_url = Option(dnf.const.BUGTRACKER)
 
     color = SelectionOption('auto', ('auto', 'never', 'always'),
@@ -805,22 +807,23 @@ class YumConf(BaseConfig):
     color_update_remote = Option('normal')
     color_search_match = Option('bold')
 
-    sslcacert = Option()
-    sslverify = BoolOption(True)
-    sslclientcert = Option()
-    sslclientkey = Option()
+    sslcacert = Option() # :api
+    sslverify = BoolOption(True) # :api
+    sslclientcert = Option() # :api
+    sslclientkey = Option() # :api
     deltarpm = BoolOption(True)
 
     history_record = BoolOption(True)
     history_record_packages = ListOption(['dnf', 'rpm'])
 
     rpmverbosity = Option('info')
-    clean_requirements_on_remove = BoolOption(False)
-    history_list_view = SelectionOption('single-user-commands',
+    strict = BoolOption(True)  # :api
+    clean_requirements_on_remove = BoolOption(True)
+    history_list_view = SelectionOption('commands',
                                         ('single-user-commands', 'users',
                                          'commands'),
-                                     mapper={'cmds'          : 'commands',
-                                             'default' :'single-user-commands'})
+                                        mapper={'cmds': 'commands',
+                                                'default': 'commands'})
 
     def dump(self):
         """Return a string representing the values of all the
@@ -916,15 +919,16 @@ class RepoConf(BaseConfig):
     ip_resolve = Inherit(YumConf.ip_resolve)
     throttle = Inherit(YumConf.throttle)
     timeout = Inherit(YumConf.timeout)
+    max_parallel_downloads = Inherit(YumConf.max_parallel_downloads)
 
     metadata_expire = Inherit(YumConf.metadata_expire)
     cost = IntOption(1000)
     priority = IntOption(99)
 
-    sslcacert = Inherit(YumConf.sslcacert)
+    sslcacert = Inherit(YumConf.sslcacert) # :api
     sslverify = Inherit(YumConf.sslverify) # :api
-    sslclientcert = Inherit(YumConf.sslclientcert)
-    sslclientkey = Inherit(YumConf.sslclientkey)
+    sslclientcert = Inherit(YumConf.sslclientcert) # :api
+    sslclientkey = Inherit(YumConf.sslclientkey)   # :api
     deltarpm = Inherit(YumConf.deltarpm)
 
     skip_if_unavailable = BoolOption(True)  # :api
